package geo;

import java.awt.Graphics;

/**
 * Courbe en polaire : r = f(t) ; x = r cos (t), y = r sin (t)
 */
public class CourbePolaire extends ObjetGeo {
	Fonction f;
	double tmin, tmax, tincrement;

/**
 * Définit la courbe à partir d'une fonction
 *
 * @param f fonction à définir
 * @param tmin valeur minimale de l'angle
 * @param tmax valeur maximale de l'angle
 * @param tincrement valeur à ajouter à l'angle à chaque étape du tracé
 */
	public CourbePolaire (Fonction f, double tmin, double tmax, double tincrement) {
		super();
		this.f = f;
		this.tmin = tmin;
		this.tmax = tmax;
		this.tincrement = tincrement;
	}


/**
 * Trace la courbe
 *
 * @param nom nom de la courbe
 * @param R repère associé au graphique
 * @param g support du tracé
 */
	public void trace (String nom, Repere R, Graphics g) {
		int Xp = Integer.MIN_VALUE;
		int Yp = Integer.MIN_VALUE;
		double ts2 = 0.5 * (tmax - tmin);
		for (double t = tmin; t < tmax; t += tincrement) {
			int X = Integer.MIN_VALUE;
			int Y = Integer.MIN_VALUE;
			double r = 0.0;
			if (f.defini(t)) {
				r = f.Image (t);
				X = R.X (r * Math.cos (t));
				Y = R.Y (r * Math.sin (t));
			} else {
				X = Integer.MIN_VALUE;
				Y = Integer.MIN_VALUE;
			}
			if ((Xp >= 0) && (Xp < R.XMAX) && (Yp >= 0) && (Yp < R.YMAX)
					&& (X >= 0) && (X < R.XMAX) && (Y >= 0) && (Y < R.YMAX)) {
				g.drawLine (Xp, Yp, X, Y);
				if ((t > ts2) && (nom != null) && (nom.length() != 0)) {
					g.drawString (nom, X + 3, Y - 3);
					ts2 = Double.MAX_VALUE;
				}
			}
			Xp = X;
			Yp = Y;
		}
	}

}
