package geo;

import java.awt.Graphics;

/**
 * Point de coordonnées (x, y) relativement à R, repère attaché à ce point.
 */
public class Pt extends ObjetGeo {
/** Abscisse du point relativement au repère attaché.*/
	public double x;
/** Ordonnée du point relativement au repère attaché.*/
	public double y;

/** Construit un point non défini.*/
	public Pt () {
		super();
	}

/** Construit une copie du point A.*/
	public Pt (Pt A) {
		super();
		x = A.x;
		y = A.y;
		defini = A.defini;
	}

/** Construit un point de position (X, Y) dans la fenêtre d'affichage relativement au repère R */
	public Pt (int X, int Y, Repere R) {
		super();
		x = R.x (X);
		y = R.y (Y);
		defini = true;
	}

/** Construit un point de coordonnées (x, y) defini ou pas */
	public Pt (double x, double y, boolean defini) {
		super();
		this.x = x;
		this.y = y;
		this.defini = defini;
	}

/** Construit un point de coordonnées (x, y) */
	public Pt (double x, double y) {
		super();
		this.x = x;
		this.y = y;
		defini = true;
	}

/** Met à jour la copie du point courant */
	public void MAJ (Pt A) {
		x = A.x;
		y = A.y;
		defini = A.defini;
	}

/** Met à jour les coordonnées du point courant */
	public void MAJ (double x, double y) {
		this.x = x;
		this.y = y;
		defini = true;
	}

/**
 * Retourne le milieu du bi-Point AB.
 * Pour cette procédure et d'autres... Cela suppose que A et B sont définis à partir d'un même repère
 */
	static public Pt milieu (Pt A, Pt B) {
		double x = 0.5 * (A.x + B.x);
		double y = 0.5 * (A.y + B.y);
		boolean defini = A.defini && B.defini;
		return new Pt (x, y, defini);
	}

/** Retourne le symétrique du point courant par rapport au Point O */
	public Pt symetrie (Pt O) {
		return new Pt ( 2.0 * O.x - x, 2.0 * O.y - y, defini && O.defini);
	}

/** Retourne le projeté du point courant sur la droite d */
	public Pt projection (Droite d) {
		boolean defini = this.defini && d.defini;
		double x = 0.0;
		double y = 0.0;
		try {
			double u = (d.a * this.x + d.b * this.y + d.c) / (d.a * d.a + d.b * d.b);
			x = this.x - u * d.a;
			y = this.y - u * d.b;
		}
		catch (ArithmeticException ae) {
			defini = false;
		}
		return new Pt (x, y, defini);
	}

/** Retourne le projeté du point courant sur un segment (le résultat peut être à l'extérieur du segment) */
	public Pt projection (Segment s) {
		double x = 0.0;
		double y = 0.0;
		double a = s.Ay - s.By;
		double b = s.Bx - s.Ax;
		defini = s.defini;
		if (defini) {
			try {
				double c = - (a * s.Ax + b * s.Ay);
				double u = (a * this.x + b * this.y + c) / (a * a + b * b);
				x = this.x - u * a;
				y = this.y - u * b;
			}
			catch (ArithmeticException ae) {
				defini = false;
			}
		}
		return new Pt (x, y, defini);
	}

/** Retourne le symétrique du point coourant par rapport à la droite d */
	public Pt symetrie (Droite d) {
		Pt p = projection (d);
		p.x = 2.0 * p.x - x;
		p.y = 2.0 * p.y - y;
		return p;
	}

/** Retourne le symetrie de A par rapport au segment s */
	public Pt symetrie (Segment s) {
		Pt p = projection (s);
		x = 2.0 * p.x - x;
		y = 2.0 * p.y - y;
		return p;
	}

/** Retourne l'image du point courant par la rotation de centre O et d'angle a (a exprimé en radians) */
	public Pt rotation (Pt O, double a) {
		double dx = x - O.x;
		double dy = y - O.y;
		double cosa = Math.cos (a);
		double sina = Math.sin (a);
		double x = O.x + cosa * dx - sina * dy;
		double y = O.y + sina * dx + cosa * dy;
		boolean defini = this.defini && O.defini;
		return new Pt (x, y, defini);
	}

/** Retourne l'image du point par la rotation de centre O et d'angle correspondant à l'arc de cercle ARC */
	public Pt rotation (Pt O, ArcDeCercle ARC) {
		return rotation (O, ARC.b - ARC.a);
	}

/** Retourne l'image du point courant par l'homothétie de centre O et de rapport k */
	public Pt homothetie (Pt O, double k) {
		return new Pt (O.x + k * (x - O.x), O.y + k * (y - O.y), defini && O.defini);
	}

/** Retourne la distance du point A au point courant */
	public double distance (Pt A) {
		if (defini && A.defini) {
			double dx = x - A.x;
			double dy = y - A.y;
			return Math.sqrt (dx * dx + dy * dy);
		}
		return Double.NaN;
	}

/** Retourne la distance du point courant à la droite d */
	public double distance (Droite d) {
		double resultat = Double.NaN;
		if (defini && d.defini) {
			try {
				resultat = Math.abs (d.a * x + d.b * y + d.c) / Math.sqrt (d.a * d.a + d.b * d.b);
			}
			catch (ArithmeticException ae) {}
		}
		return resultat;
	}

/**
* Retourne la distance du point courant au segment s.
*/
	public double distance (Segment s) {
		double resultat = Double.NaN;
		if (defini && s.defini) {
			double a = s.By - s.Ay;
			double b = s.Ax - s.Bx;
			if ((a != 0.0) || (b != 0.0))
				resultat = Math.abs (a * x + b * y + s.Ay * s.Bx - s.Ax * s.By) / Math.sqrt (a * a + b * b);
		}
		return resultat;
	}

/** Retourne, s'il existe, le point d'intersection des droites d1 et d2 */
	static public Pt intersection (Droite d1, Droite d2) {
		double det = d1.a * d2.b - d2.a * d1.b;
		boolean defini = (d1.defini) && (d2.defini) && (det != 0.0);
		double x = 0.0;
		double y = 0.0;
		if (defini) {
			x = (d2.c * d1.b - d1.c * d2.b) / det;
			y = (d2.a * d1.c - d1.a * d2.c) / det;
		}
		return new Pt (x, y, defini);
	}

/** Retourne l'image du point courant par la translation de vecteur v */
	public Pt translation (Vecteur v) {
		double x = 0.0;
		double y = 0.0;
		boolean defini = this.defini && v.defini;
		if (defini) {
			x = this.x + v.x;
			y = this.y + v.y;
		}
		return new Pt (x, y, defini);
	}

/** Retourne les deux points d'intersection, s'ils existent, de la droite d et du cercle c */
	static public Pt[] intersection (Droite d, Cercle c) {
		Pt[] p = new Pt [2];
		p[0] = i12 (d, c, 1.0);
		p[1] = i12 (d, c, -1.0);
		return p;
	}

	static private Pt i12 (Droite d, Cercle c, double signe) {
		boolean defini = (d.defini) && (c.defini);
		double x = 0.0;
		double y = 0.0;
		if (defini) {
			if (d.b != 0.0) {
				double u = d.a * (d.c + d.b * c.Oy) - d.b * d.b * c.Ox;
				double v = d.a * d.a + d.b * d.b;
				double w = d.c + d.b * c.Oy;
				double deltap = u * u - v * (d.b * d.b * (c.Ox * c.Ox - c.r * c.r) + w * w);
				if (deltap >= 0.0) {
					x = (- u + signe * Math.sqrt (deltap)) / v;
					y = - (d.a * x + d.c) / d.b; }
				else
					defini = false;
			} else {
				x = - d.c / d.a;
				double u = x - c.Ox;
				double v = c.r * c.r - u * u;
				if (v >= 0.0)
					y = c.Oy + signe * Math.sqrt (v);
				else
					defini = false;
			}
		}
		return new Pt (x, y, defini);
	}

/** Retourne les deux points d'intersection, s'ils existent, des deux cercles c1 et c2 */
	static public Pt[] intersection (Cercle c1, Cercle c2) {
		Pt[] p = new Pt [2];
		p[0] = icc (c1, c2, 1.0);
		p[1] = icc (c1, c2, -1.0);
		return p;
	}

	static private Pt icc (Cercle c1, Cercle c2, double signe) {
		double x = 0.0;
		double y = 0.0;
		boolean defini;
		if (defini = c1.defini && c2.defini) {
			double a1 = -2.0 * c1.Ox;
			double b1 = -2.0 * c1.Oy;
			double a2 = -2.0 * c2.Ox;
			double b2 = -2.0 * c2.Oy;
			double a12 = a1 - a2;
			double b12 = b1 - b2;
			double a = a12 * a12 + b12 * b12;
			if (defini = (a != 0.0)) {
				double C1 = c1.Ox * c1.Ox + c1.Oy * c1.Oy - c1.r * c1.r;
				double C2 = c2.Ox * c2.Ox + c2.Oy * c2.Oy - c2.r * c2.r;
				double C12 = C1 - C2;
				double b, c;
				boolean bpga = Math.abs(b12) >= Math.abs(a12);
				if (bpga) {
					b = 2.0 * a12 * C12 + b12 * (b1 * a2 - a1 * b2);
					c = C12 * C12 + b12 * (b1 * C2 - b2 * C1);
				} else {
					b = 2.0 * b12 * C12 + a12 * (a1 * b2 - b1 * a2);
					c = C12 * C12 + a12 * (a1 * C2 - a2 * C1);
				}
				double delta = b * b - 4.0 * a * c;
				if (defini = (delta >= 0.0)) {
					x = -0.5 * (b + signe * Math.sqrt (delta)) / a;
					if (bpga)
						y = - (a12 * x + C12) / b12;
					else {
						y = x; x = - (b12 * x + C12) / a12;
					}
				}
			}
		}
		return new Pt (x, y, defini);
	}

/** Retourne le centre du cercle c */
	static Pt centre (Cercle c) {
		double x = 0.0;
		double y = 0.0;
		boolean defini = c.defini;
		if (defini) {
			x = c.Ox;
			y = c.Oy;
		}
		return new Pt (x, y, defini);
	}

/**
 * Trace le point courant
 *
 * @param nom nom du point
 * @param R repère associé
 * @param g support graphique
 */
	public void trace (String nom, Repere R, Graphics g) {
		int X = R.X (x);
		int Y = R.Y (y);
		if (defini) {
			g.fillRect (X - 1, Y - 1, 3, 3);
			if (nom != null && nom.length() != 0)
				g.drawString (nom, X + 3, Y - 3);
		}
	}

/**
 * Trace uniquement le nom du point courant
 *
 * @param nom nom du point
 * @param R repère associé
 * @param g support graphique
 */
	public void traceNom (String nom, Repere R, Graphics g) {
		int X = R.X (x);
		int Y = R.Y (y);
		if (defini && nom != null && (nom.length() != 0))
			g.drawString (nom, X + 3, Y - 3);
	}

/** toString */
	public String toString() {
		return this.getClass().getName() + " : (" + x + ", "+ y + " ) defini = " + defini;
	}
}
